#!/usr/bin/python3

"""
$Id: dataset_details.py 420 2018-02-21 20:55:12Z heijer $
$Date: 2018-02-21 21:55:12 +0100 (wo, 21 feb 2018) $
$Author: heijer $
$Revision: 420 $
$HeadURL: https://repos.deltares.nl/repos/kustgenese2/trunk/datamanagement/datamanagementplan/contrib/dataset_details.py $
"""

# External modules
import os
import configparser
import subprocess
import xml.etree.ElementTree as ET
import collections
import argparse

# Own modules
import dataset_overview
import subversion
import utilsRepo as utl


# GLOBALS
CFG_FILENAME = 'dataset_details.cfg'  # standard name of track change files


def write_details(url, cfgfile=CFG_FILENAME):

    DS = dataset_overview.DataSetInfo(url)
    cfg = configparser.ConfigParser(dict_type=collections.OrderedDict)
    if os.path.exists(cfgfile):
        cfg.read(cfgfile)

    # Setting all
    if not cfg.has_section('general'):
        cfg.add_section('general')
    cfg.set('general', 'dataset_name', DS.name)
    cfg.set('general', 'dataset_url', DS.url)
    cfg.set('general', 'domain_name', DS.domain_name)
    cfg.set('general', 'domain_url', DS.domain_url)
    cfg.set('general', 'date', DS.date)
    cfg.set('general', 'revision', DS.revision)
    cfg.set('general', 'readme', str(DS.has_readme()))
    cfg.set('general', 'raw', str(DS.has_raw()))
    cfg.set('general', 'scripts', str(DS.has_scripts()))
    cfg.set('general', 'authors', ", ".join(DS.authors(unique=True, order_by='name')))
    cfg.set('general', 'size', utl.size2hr(DS.total_size))
    if 'contact' not in cfg['general']:
        cfg.set('general', 'contact', '')
    if 'owner' not in cfg['general']:
        cfg.set('general', 'owner', '')

    if DS.has_raw():
        if not cfg.has_section('raw'):
            cfg.add_section('raw')
        revs, authors, dates, msgs = zip(*dataset_overview.log(DS.raw_url))
        cfg.set('raw', 'date', dates[0])
        cfg.set('raw', 'revision', str(revs[0]))
        cfg.set('raw', 'authors', ", ".join(sorted(list(set(authors)))))
        cfg.set('raw', 'size', dataset_overview.size2hr(DS.raw_size))
        cfg.set('raw', 'extensions', ", ".join(sorted(list(set(DS.raw_exts)))))
        cfg.set('raw', 'url', DS.raw_url)

    if DS.has_scripts():
        if not cfg.has_section('scripts'):
            cfg.add_section('scripts')
        revs, authors, dates, msgs = zip(*dataset_overview.log(DS.scripts_url))
        cfg.set('scripts', 'date', dates[0])
        cfg.set('scripts', 'revision', str(revs[0]))
        cfg.set('scripts', 'authors', ", ".join(sorted(list(set(authors)))))
        cfg.set('scripts', 'extensions', ", ".join(sorted(list(set(DS.script_exts)))))
        cfg.set('scripts', 'url', DS.scripts_url)

    # Order the content of each section alphabetically
    for section in cfg._sections:
        cfg._sections[section] = collections.OrderedDict(
            sorted(cfg._sections[section].items(), key=lambda t: t[0]))

    with open(cfgfile, 'w') as fobj:
        cfg.write(fobj)





def main(force=False):
    stdout = subprocess.check_output('svn info "%s"  | awk \'/^Working Copy Root Path:/{ print $NF}\'' % os.path.dirname(os.path.abspath(__file__)), shell=True)

    wc_root = stdout.decode('utf-8').strip()

    wc_root_url = path2url(wc_root)

    # TODO: check if wc_root_url is at trunk level or higher

    stdout = subprocess.check_output('svn ls -R "%s"' % wc_root_url, shell=True)

    lst = stdout.decode('utf-8').strip().split()
    cfgfiles = []
    datasets = []
    for item in lst:
        if item.endswith('raw/') or item.endswith(CFG_FILENAME):
            tail = os.path.dirname(item[:-1])
            dataset_url = os.path.join(wc_root_url, tail)
            if dataset_url in datasets:
                continue
            datasets.append(dataset_url)
            dir_splits = tail.split(os.path.sep)
            for i, d in enumerate(dir_splits):
                p = os.path.join(wc_root, os.path.sep.join(dir_splits[:(i+1)]))
                if not os.path.exists(p):
                    os.system('svn up --set-depth empty "%s"' % p)
            dataset_dir = os.path.join(wc_root, tail)
            cfgfile = os.path.join(dataset_dir, CFG_FILENAME)
            make_cfg = force
            if CFG_FILENAME in svn_list(dataset_url).split():
                os.system('svn up "%s"' % cfgfile)
                cfg_rev = svn_info(os.path.join(dataset_url, CFG_FILENAME))
                dataset_rev = svn_info(dataset_url)
                if dataset_rev > cfg_rev:
                    make_cfg = True
            else:
                make_cfg = True

            if make_cfg:
                print(dataset_url)

                write_details(dataset_url, cfgfile=cfgfile)
                cfgfiles.append(cfgfile)
                if svn_status(cfgfile) == 'unversioned':
                    os.system('svn add "%s"' % cfgfile)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Populate %s files per (sub) dataset with actual version information.'
                                                 % CFG_FILENAME)
    parser.add_argument('-f', '--force', action='store_true',
                        help='force updating the %s files'
                             % CFG_FILENAME)
    args = parser.parse_args()
    main(force=args.force)
